#include<ssslib.h>
#include<stdio.h>
#include<stdlib.h>
#include<stdlib.h>

struct mangling_zlib_state {
  int sss_errno;
  int debug;
};

void *mangling_zlib_LTX_mangling_getstate(SSS_ConfigDict cfg,int debug){
  struct mangling_zlib_state *ret;

  if (debug)
    printf("mangling_zlib: starting getstate\n");

  ret = malloc(sizeof(struct mangling_zlib_state));

  ret->debug = debug;

  if (debug)
    printf("mangling_zlib: successful getstate\n");

  return ret;
}

char *mangling_zlib_LTX_mangle_message(void *mangling_state,char *buf,int incount,int *outcount){
  int rc;
  int msglen;
  int len_msglen;
  char *newsendbuf;
  unsigned char *compbuf;
  unsigned long complen;
  char snbuf[1];
  struct mangling_zlib_state *state = mangling_state;

  if (state->debug)
    printf("mangling_zlib: start mangle_message\n");

  complen = (int)(1.1 * (float)msglen) + 12;
  compbuf = malloc(complen);
  rc = compress(compbuf,&complen,(unsigned char *)buf,incount);
  if (rc){
    if (state->debug)
      printf("mangling_zlib: compression error\n");
    state->sss_errno = SSS_PROT_ERROR;
    return NULL;
  }
  len_msglen = snprintf(snbuf,1,"%i ",incount);
  newsendbuf = malloc(len_msglen + complen);
  sprintf(newsendbuf,"%i ",incount);
  bcopy(compbuf,newsendbuf+len_msglen,complen);

  *outcount = complen + len_msglen;

  if (state->debug)
    printf("mangling_zlib: successful mangle_message\n");
  
  return newsendbuf;
}

char *mangling_zlib_LTX_unmangle_message(void *mangling_state,char *buf,int incount,int *outcount){
  char *endbuf;
  int uncompsize;
  char *retbuf;
  int offset;
  char snbuf[1];
  int rc;
  struct mangling_zlib_state *state = mangling_state;

  if (state->debug)
    printf("mangling_zlib: start unmangle_message\n");

  endbuf = malloc(incount + 1);
  bcopy(buf,endbuf,incount);
  endbuf[incount] = '\0';

  sscanf(endbuf,"%i ",&uncompsize);
  offset = snprintf(snbuf,1,"%i ",uncompsize);
  retbuf = malloc(uncompsize);
  *outcount = uncompsize;
  rc = uncompress(retbuf,outcount,buf+offset,incount-offset);
  if (rc){
    if (state->debug)
      printf("mangling_zlib: compression error\n");
    state->sss_errno = SSS_PROT_ERROR;
    return NULL;
  }
  free(endbuf);

  if (state->debug)
    printf("mangling_zlib: successful unmangle_message\n");
  
  return retbuf;
} 

int mangling_zlib_LTX_mangling_close(void *state){
  free(state);
  return 0;
}
