#include<ssslib.h>
#include<stdlib.h>

#include<errno.h>
#include<netinet/in.h>
#include<sys/socket.h>
#include<netdb.h>

struct transport_tcp_state {
  int sss_errno;
  int debug;
  int socket;
};

void *transport_tcp_LTX_transport_getstate(SSS_ConfigDict cfg,int debug){
  struct transport_tcp_state *ret;

  if (debug)
    printf("transport_tcp: starting init\n");

  ret = malloc(sizeof(struct transport_tcp_state));
  ret->debug = debug;

  if (debug)
    printf("transport_tcp: successful init\n");

  return ret;
}

int transport_tcp_LTX_serv_connect(void *transport_state,int extra){
  char *next;
  struct transport_tcp_state *state = transport_state;
  if (state->debug)
    printf("transport_tcp: starting serv_connect\n");

  state->socket = extra;

  if (state->debug)
    printf("transport_tcp: successful serv_connect\n");
  return 0;
}

int transport_tcp_LTX_cli_connect(void *transport_state,char *host,int extra){
  int port;
  char *next;
  int rc;
  struct hostent *ipaddr;
  struct sockaddr_in addr;
  struct transport_tcp_state *state = transport_state;
  
  if (state->debug)
    printf("transport_tcp: starting cli_connect\n");
  
  port = extra;

  state->socket = socket(PF_INET,SOCK_STREAM,0);
  if (state->socket == -1){
    switch(errno){
    case EPROTONOSUPPORT:
      state->sss_errno = SSS_PROTO_NO_SUPPORT;
      break;
    case EAFNOSUPPORT:
      state->sss_errno = SSS_FAM_NO_SUPPORT;
      break;
    case ENFILE:
    case ENOBUFS:
    case ENOMEM:
      state->sss_errno = SSS_NOMEM;
      break;
    case EACCES:
      state->sss_errno = SSS_NOSOCKPERM;
      break;
    case EINVAL:
      state->sss_errno = SSS_INVALPROT;
      break;
    default:
      if (state->debug)
	printf("transport_tcp: cli_connect socket call unhandled error\n");
      state->sss_errno = SSS_UNKNOWN_ERROR;
    }
    if (state->debug)
      printf("transport_tcp: cli_connect failed\n");
    return -1;
  }

  ipaddr = gethostbyname(host);

  if(!ipaddr){
    switch (h_errno) {
    case HOST_NOT_FOUND:
      state->sss_errno = SSS_HOST_NOT_FOUND;
      break;
    case NO_ADDRESS:
      state->sss_errno = SSS_NO_ADDRESS;
      break;
    case NO_RECOVERY:
      state->sss_errno = SSS_UNRECOVERABLE_DNS_ERROR;
      break;
    case TRY_AGAIN:
      state->sss_errno = SSS_TRY_AGAIN;
      break;
    default:
      if (state->debug)
	printf("transport_tcp: cli_connect gethostbyname call unhandled error\n");
      state->sss_errno = SSS_UNKNOWN_ERROR;
    }
    if (state->debug)
      printf("transport_tcp: cli_connect failed\n");
    return -1;
  }

  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  memcpy(&(addr.sin_addr.s_addr),*(ipaddr->h_addr_list),sizeof(struct in_addr));
  memset(&(addr.sin_zero),'\0',8);
 preconnect:
  rc = connect(state->socket,(struct sockaddr *)&addr,sizeof(struct sockaddr));
  
  if (rc == -1) {
    switch (errno){
    case EBADF:
      state->sss_errno = SSS_BAD_FD;
      break;
    case EFAULT:
      state->sss_errno = SSS_FAULT;
      break;
    case ENOTSOCK:
      state->sss_errno = SSS_BAD_FD;
      break;
    case EISCONN:
      state->sss_errno = SSS_ALREADY_CONNECTED;
      break;
    case ECONNREFUSED:
      state->sss_errno = SSS_CONN_REFUSED;
      break;
    case ETIMEDOUT:
      state->sss_errno = SSS_TIMEOUT;
      break;
    case ENETUNREACH:
      state->sss_errno = SSS_NET_UNREACH;
      break;
    case EHOSTUNREACH:
      state->sss_errno = SSS_NO_ROUTE;
      break;
    case EADDRINUSE:
      state->sss_errno = SSS_ADDR_IN_USE;
      break;
    case EAGAIN:
      state->sss_errno = SSS_NO_FREE_PORTS;
      break;
    case ECONNRESET:
      state->sss_errno = SSS_CONN_RESET;
      break;
    case EACCES:
    case EPERM:
      state->sss_errno = SSS_NO_CONNECT_PERM;
      break;
    case EINTR:
      goto preconnect;
      break;
    default:
      if (state->debug)
	printf("transport_tcp: cli_connect connect call unhandled error\n");
      state->sss_errno = SSS_UNKNOWN_ERROR;
    }
    if (state->debug)
      printf("transport_tcp: cli_connect failed\n");
    return -1;
  }
  
  if (state->debug)
    printf("transport_tcp: successful cli_connect\n");

  return 0;
}

int transport_tcp_LTX_transport_write(void *transport_state,void *buf,int count){
  int rc;

  struct transport_tcp_state *state = transport_state;

  if (state->debug)
    printf("transport_tcp: starting write\n");

  rc = write(state->socket,buf,count);

  if (rc == -1){
    switch(errno){
    case EBADF:
      state->sss_errno = SSS_BAD_FD;
      break;
    case ENOTSOCK:
      state->sss_errno = SSS_NOT_A_SOCKET;
      break;
    case EFAULT:
      state->sss_errno = SSS_FAULT;
      break;
    case EMSGSIZE:
      state->sss_errno = SSS_INVALID_SIZE;
      break;
    case ENOBUFS:
      state->sss_errno = SSS_NOMEM;
      break;
    case EINTR:
      state->sss_errno = SSS_INTERRUPT;
      break;
    case ENOMEM:
      state->sss_errno = SSS_NOMEM;
      break;
    case EINVAL:
      state->sss_errno = SSS_INVAL_ARG;
      break;
    case EPIPE:
      state->sss_errno = SSS_BROKEN_PIPE;
      break;
    case ECONNRESET:
      state->sss_errno = SSS_CONN_RESET;
      break;
    default:
      if (state->debug)
	printf("transport_tcp: write call unhandled error\n");
      state->sss_errno = SSS_UNKNOWN_ERROR;
    }
    return -1;
  }

  if (state->debug)
    printf("transport_tcp: successful write\n");
  
  return rc;
}

int transport_tcp_LTX_transport_writen(void *transport_state,void *buf,int count){
  int nleft, nwritten;
  struct transport_tcp_state *state = transport_state;

  if (state->debug)
    printf("transport_tcp: starting writen\n");

  nleft = count;
  while (nleft > 0)   {
    nwritten = transport_tcp_LTX_transport_write(transport_state, buf, nleft);
    if (nwritten <= 0)
      return -1;
    nleft -= nwritten;
    buf += nwritten;
  }
  if (state->debug)
    printf("transport_tcp: successful writen\n");
  return 0;
}

int transport_tcp_LTX_transport_read(void *transport_state,void *buf,int count){
  int rc;
  struct transport_tcp_state *state = transport_state;

  if (state->debug)
    printf("transport_tcp: starting read\n");

  rc = read(state->socket,buf,count);

  if (rc == -1){
    switch(errno){
    case EBADF:
      state->sss_errno = SSS_BAD_FD;
      break;
    case ECONNREFUSED:
      state->sss_errno = SSS_CONN_REFUSED;
      break;
    case ENOTCONN:
      state->sss_errno = SSS_SOCK_NOT_CONNECTED;
      break;
    case EINTR:
      state->sss_errno = SSS_INTERRUPT;
      break;
    case EFAULT:
      state->sss_errno = SSS_FAULT;
      break;
    case EINVAL:
      state->sss_errno = SSS_INVAL_ARG;
      break;
    case ECONNRESET:
      state->sss_errno = SSS_CONN_RESET;
      break;
    case ESPIPE:
      state->sss_errno = SSS_ILLEGAL_SEEK;
      break;
    default:
      if (state->debug)
	printf("transport_tcp: read call unhandled error\n");
      state->sss_errno = SSS_UNKNOWN_ERROR;
    }
    return -1;
  }

  if (rc == 0){
    state->sss_errno = SSS_EOF;
    return -1;
  }
    
  if (state->debug)
    printf("transport_tcp: successful read\n");
  
  return rc;
}

int transport_tcp_LTX_transport_readn(void *transport_state,void *buf,int count){
  int nleft, nread;
  struct transport_tcp_state *state = transport_state;
  
  if (state->debug)
    printf("transport_tcp: starting readn\n");

  nleft = count;
  while (nleft > 0){
    nread = transport_tcp_LTX_transport_read(transport_state, buf, nleft);
    if (nread < 0)
      return nread;
    else if (nread == 0)
      break;
    nleft -= nread;
    buf += nread;
  }

  if (state->debug)
    printf("transport_tcp: successful readn\n");

  return 0;

}

int transport_tcp_LTX_transport_close(void *transport_state){
  int debug;
  struct transport_tcp_state *state = transport_state;
  
  if (state->debug)
    printf("transport_tcp: starting transport_close\n");

  close(state->socket);
  debug = state->debug;
    
  free(state);

  if (debug)
    printf("transport_tcp: successful transport_close\n");

  return 0;
}
